"use strict";
/**
 * GitDocumentDB plugin for remote connection using NodeGit
 * Copyright (c) Hidekazu Kubota
 *
 * This source code is licensed under the Mozilla Public License Version 2.0
 * found in the LICENSE file in the root directory of this source tree.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.clone = exports.sleep = void 0;
const nodegit_1 = __importDefault(require("@sosuisen/nodegit"));
const tslog_1 = require("tslog");
const error_1 = require("./error");
const const_1 = require("./const");
const authentication_1 = require("./authentication");
const net_1 = require("./net");
function sleep(msec) {
    return new Promise(resolve => setTimeout(resolve, msec));
}
exports.sleep = sleep;
async function clone(workingDir, remoteOptions, logger) {
    logger !== null && logger !== void 0 ? logger : (logger = new tslog_1.Logger({
        name: 'clone',
        minLevel: 'trace',
        displayDateTime: false,
        displayFunctionName: false,
        displayFilePath: 'hidden',
    }));
    if (remoteOptions !== undefined &&
        remoteOptions.remoteUrl !== undefined &&
        remoteOptions.remoteUrl !== '') {
        /**
         * Retry if network errors.
         */
        let result = {
            ok: false,
        };
        let retry = 0;
        for (; retry < const_1.NETWORK_RETRY; retry++) {
            // eslint-disable-next-line no-await-in-loop
            result = await net_1.checkHTTP(remoteOptions.remoteUrl, const_1.NETWORK_TIMEOUT).catch(err => err);
            if (result.ok) {
                break;
            }
            else {
                logger.debug(`NetworkError in cloning: ${remoteOptions.remoteUrl}, ` + result);
            }
            // eslint-disable-next-line no-await-in-loop
            await sleep(const_1.NETWORK_RETRY_INTERVAL);
        }
        if (!result.ok) {
            // Set retry number for code test
            throw new error_1.Err.CannotConnectError(retry, remoteOptions.remoteUrl, result.toString());
        }
        return await nodegit_1.default.Clone.clone(remoteOptions.remoteUrl, workingDir, {
            fetchOpts: {
                callbacks: authentication_1.createCredential(remoteOptions),
            },
        }).catch(err => {
            // Errors except CannotConnectError are handled in sync().
            logger.debug(`Error in cloning: ${remoteOptions.remoteUrl}, ` + err);
            // The db will try to create remote repository in sync() if 'undefined' is returned.
            return undefined;
        });
    }
    return undefined;
}
exports.clone = clone;
//# sourceMappingURL=remote-nodegit.js.map